#import "../ImageOpr.h"
#import <appkit/Application.h>
#import <appkit/publicWraps.h>
#import <appkit/NXBitmapImageRep.h>
#import <appkit/Control.h>
#import <appkit/Panel.h>
#import <appkit/TextField.h>
#import <stdio.h>
#import <stdlib.h>
#import <string.h>
#import <libc.h>
#import <streams/streams.h>
#import "../TVController.h"
#import "../ToyWin.h"
#import "../ToyWinEPS.h"
#import "../ToyView.h"
#import "../common.h"
#import "../getpixel.h"
#import "imageOperation.h"


@implementation ImageOpr (ColorReplace)


#define  CLR_THRESHOLD	(32 * 32 * 3)
#define  BRT_THRESHOLD	(64 * 3)
#define  MONO_THRESHOLD	128

static BOOL isMono;
static const int *origClr;
static int origbrite, cthresh, bthreshm, bthreshp;

static void initNearColor(const int *orig, float comparison, int cnum)
{
    origClr = orig;
    if (cnum < 3) {
	isMono = YES;
	origbrite = orig[0];
	bthreshm = orig[0] - MONO_THRESHOLD * comparison;
	bthreshp = orig[0] + MONO_THRESHOLD * comparison;
    }else {
	isMono = NO;
	origbrite = Bright255(orig[RED], orig[GREEN], orig[BLUE]);
	bthreshm = origbrite - BRT_THRESHOLD * comparison;
	bthreshp = origbrite + BRT_THRESHOLD * comparison;
	cthresh = CLR_THRESHOLD * comparison;
    }
}

static BOOL isNearColor(const int *elm)
{
	int i, brt;
	double dif, d, fr;

	if (isMono)
		return (elm[0] < bthreshp && elm[0] > bthreshm);

	brt = Bright255(elm[RED], elm[GREEN], elm[BLUE]);
	if (brt > bthreshp || brt < bthreshm)	/* compare brightness */
		return NO;
	if (brt == 0 || brt == 255 * 3)
		return YES;	/* White/Black within allowed brightness */
	fr = (float)origbrite / brt;
	for (dif = 0, i = 0; i < 3; i++) {
		d = fr * elm[i] - origClr[i];
		dif += d * d;
	}
	if (dif < cthresh)
		return YES;
	fr = (255.0 * 3 - origClr[GREEN]) / (255.0 * 3 - elm[GREEN]);
	for (dif = 0, i = 0; i < 3; i++) {
		d = fr * (255.0 - elm[i]) - (255.0 - origClr[i]);
		dif += d * d;
	}
	if (dif < cthresh)
		return YES;
	return NO;
}

static commonInfo *makeClrChangeMap(const int *origclr, const int *newclr,
	float comparison, BOOL uniform, BOOL outside,
	commonInfo *cinf, NXRect *selrect,
	const unsigned char *map[], unsigned char *newmap[])
{
	commonInfo *newinf = NULL;
	int pn, alp, cn;
	int x, y, i;
	int xorg=0, yorg=0, xend=0, yend=0;

	newmap[0] = NULL;
	if (selrect == NULL
		|| selrect->size.width <= 0 || selrect->size.height <= 0
		|| (selrect->size.width >= cinf->width &&
			selrect->size.height >= cinf->height))
		selrect = NULL;
	else {
		xorg = selrect->origin.x;
		xend = selrect->size.width + xorg - 1;
		yorg = cinf->height -
			(selrect->origin.y + selrect->size.height);
		yend = cinf->height - selrect->origin.y - 1;
	}
	if ((newinf = (commonInfo *)malloc(sizeof(commonInfo))) == NULL)
		goto ErrEXIT;
	*newinf = *cinf;
	newinf->palette = NULL;
	newinf->palsteps = 0;
	newinf->isplanar = YES;
	newinf->bits = 8;
	newinf->xbytes = newinf->width;
	if (cinf->alpha && origclr[ALPHA] == AlphaTransp && selrect == NULL)
		newinf->alpha = NO;
	if (newclr[ALPHA] == AlphaTransp)
		newinf->alpha = YES;
	if (cinf->numcolors == 1) {
		if (newclr[RED] != newclr[GREEN]
				|| newclr[RED] != newclr[BLUE]) {
			newinf->numcolors = 3;
			newinf->cspace = NX_RGBColorSpace;
		}else
			newinf->cspace = NX_OneIsWhiteColorSpace;
	}
	pn = cn = (newinf->numcolors == 1) ? 1 : 3;
	if (newinf->alpha) alp = pn++;
	else alp = 0;
	initNearColor(origclr, comparison, cn);
	if (allocImage(newmap, newinf->width, newinf->height, 8, pn))
		goto ErrEXIT;
	if (initGetPixel(cinf) != 0)
		goto ErrEXIT;
	resetPixel(map, 0);
	for (y = 0; y < newinf->height; y++) {
	    int elm[MAXPLANE], gen[MAXPLANE];
	    const int *cp;
	    BOOL selflag = selrect ? (yorg <= y && y <= yend) : NO;
	    int ptr = newinf->width * y;
	    for (x = 0; x < newinf->width; x++, ptr++) {
		getPixel(&elm[RED], &elm[GREEN], &elm[BLUE], &elm[ALPHA]);
		if (selrect == NULL ||
			(outside ^ (selflag && xorg <= x && x <= xend))) {
		    if (origclr[ALPHA] == AlphaTransp)
			cp = (elm[ALPHA] == AlphaTransp) ? newclr : elm;
		    else if (elm[ALPHA] == AlphaTransp)
			cp = elm;	/* origclr[ALPHA] != AlphaTransp */
		    else if (comparison == 0.0) {
			for (i = 0; i < cn; i++)
				if (origclr[i] != elm[i]) break;
			cp = (i >= cn) ? newclr : elm;
		    }else if (isNearColor(elm)) {
				if (uniform) cp = newclr;
				else {
				    int d;
				    for (i = 0; i < cn; i++) {
					d = elm[i] - origclr[i] + newclr[i];
					gen[i] = (d>255)? 255: ((d<0)? 0: d);
				    }
				    if (alp) gen[ALPHA] = newclr[ALPHA];
				    cp = gen;
				}
		    }else
		    	cp = elm;
		}else
		    cp = elm;
		for (i = 0; i < cn; i++)
		    newmap[i][ptr] = cp[i];
		if (alp)
		    newmap[alp][ptr] = cp[ALPHA];
	    }
	}
	return newinf;

ErrEXIT:
	if (newinf)
		free((void *)newinf);
	if (newmap[0]) free((void *)newmap[0]);
	return NULL;
}


- colorChange:(int *)origclr to:(int *)newclr with:(float)comparison
		uniformly:(BOOL)uniform area:(BOOL)outside
{
	ToyWin	*tw, *newtw = nil;
	ToyView	*tv = nil;
	commonInfo	*cinf, *newinf;
	const char *fnam;
	char fn[256];
	NXBitmapImageRep *rep;
	unsigned char *map[MAXPLANE], *newmap[MAXPLANE];
	int i;

	for (i = 0; i < 4; i++)
		if (origclr[i] != newclr[i]) break;
	if (i >= 4 /* Same Color */
		|| (tw = [theController keyWindow]) == nil) {
		NXBeep();
		return self;
	}
	tv = [tw toyView];
	cinf = [tv commonInfo];
	if (!cinf->alpha && origclr[ALPHA] == AlphaTransp) {
		NXBeep();
		return self;
	}
	fnam = [tw filename];
	if (![self checkCommonInfo:cinf filename:fnam])
		return self;
	sprintf(fn, "%s(ColorChange)", fnam);
	rep = (NXBitmapImageRep *)[[tv image] bestRepresentation];
	[rep getDataPlanes: map];
	newinf = makeClrChangeMap(origclr, newclr, comparison,
		uniform, outside, cinf, [tv selectedScaledRect], map, newmap);
	if (newinf == NULL) {
		warnAlert(fnam, Err_MEMORY);
		return self;
	}
	newtw = [[ToyWin alloc] init:tw by:ColorChange];
	[newtw initLocateWindow:fn width:newinf->width height:newinf->height];
	[newtw makeComment: newinf from: cinf];
	[newtw drawView:newmap info: newinf];

	[theController newWindow: newtw];
	return self;
}


static int color_tone(const commonInfo *cinf,
		float degree, unsigned char **planes)
{
	int total, i, idx;
	int pnum, v;
	int elm[MAXPLANE];
	int alp, err;
	float	avr;

	pnum = cinf->numcolors;	/* must be 3 */
	if (cinf->alpha) alp = pnum++;
	else alp = 0;
	err = allocImage(planes, cinf->width, cinf->height, 8, pnum);
	if (err) return err;

	total = cinf->height * cinf->width;
	for (idx = 0; idx < total; idx++) {
		getPixel(&elm[0], &elm[1], &elm[2], &elm[3]);
		avr = (elm[0] + elm[1] + elm[2]) / 3.0;
		for (i = 0; i < 3; i++) {
			v = (elm[i] - avr) * degree + avr;
			planes[i][idx] = (v >= 255) ? 255 : ((v <= 0) ? 0 : v);
		}
		if (alp) planes[alp][idx] = elm[3];
	}
	return 0;
}

/* Local Method */
- (int)doEnhance:(const char *)fn info:(const commonInfo *)cinf
	parent:(ToyWin *)parent with:(float)degree
{
	ToyWin *tw;
	int	err;
	commonInfo *newinf = NULL;
	unsigned char *working[MAXPLANE];

	working[0] = NULL;
	tw = NULL;
	if ((newinf = (commonInfo *)malloc(sizeof(commonInfo))) == NULL)
		goto ErrEXIT;
	*newinf = *cinf;
	newinf->bits = 8;
	newinf->isplanar = YES;
	newinf->xbytes = newinf->width;
	newinf->palsteps = 0;
	newinf->palette = NULL;
	[theController messageDisplay: "Enhancing..."];
	tw = [[ToyWin alloc] init:parent by:ColorTone];
	[tw initLocateWindow:fn width:newinf->width height:newinf->height];

	err = color_tone(cinf, degree, working);
	[theController messageDisplay:NULL];
	if (err) goto ErrEXIT;
	[tw makeComment: newinf from: cinf];
	if ([tw drawView:working info: newinf] == nil)
		goto ErrEXIT;
	[theController newWindow: tw];

	return 0;

ErrEXIT:
	if (working[0]) free((void *)working[0]);
	if (newinf) free((void *)newinf);
	if (tw) [tw free];
	return Err_MEMORY;
}


- colorToneEnhance:(float)degree
{
	ToyWin	*tw;
	ToyView	*tv = NULL;
	NXImageRep	*rep;
	commonInfo	*cinf;
	unsigned char	*map[MAXPLANE];
	char	fn[MAXFILENAMELEN];
	const char *fnam;
	int	err;

	if (degree == 1.0 || (tw = [self keyParentWindow:ColorTone]) == nil) {
		NXBeep();
		return self;
	}
	tv = [tw toyView];
	fnam = [tw filename];
	cinf = [tv commonInfo];
	if (cinf->numcolors == 1) {
		warnAlert(fnam, Err_OPR_IMPL);
		return self;
	}
	if (![self checkCommonInfo:cinf filename:fnam])
		return self;

	rep = [[tv image] bestRepresentation];
	[(NXBitmapImageRep *)rep getDataPlanes: map];
	if ((err = initGetPixel(cinf)) != 0) {
		errAlert(fnam, err);
		return self;
	}
	resetPixel(map, 0);
	sprintf(fn, "%s(ColorTone)", fnam);
	if ((err = [self doEnhance:fn info:cinf parent:tw with:degree]) != 0)
		errAlert(fnam, err);
	return self;
}


@end
